function [L,V,D,B,xD,xB,dMxdt] = colsys(X,U,stab_d,no)

% Parameters
parameter

% Disturbances
dvar

% Intervals for the "states"
stab_ntnc = p.NC.stab*p.NT.stab + 1;
ben_ntnc  = p.NC.ben*p.NT.ben   + 1;
tol_ntnc  = p.NC.tol*p.NT.tol   + 1;

% Splitting the states and parameters
stab_x = [X(1:stab_ntnc); U(1:4)];
ben_x  = [X(stab_ntnc+1:stab_ntnc+ben_ntnc); U(5:8)];
tol_x  = [X(stab_ntnc+ben_ntnc+1:end); U(9:end)];

stab_p.ntnfnc = [p.NT.stab p.NF.stab p.NC.stab]; stab_p.alpha = p.alpha.stab; stab_p.Kf = p.Kf.stab; stab_p.Muw = p.Muw.stab;
ben_p.ntnfnc  = [p.NT.ben p.NF.ben p.NC.ben];    ben_p.alpha  = p.alpha.ben;  ben_p.Kf  = p.Kf.ben;  ben_p.Muw  = p.Muw.ben;
tol_p.ntnfnc  = [p.NT.tol p.NF.tol p.NC.tol];    tol_p.alpha  = p.alpha.tol;  tol_p.Kf  = p.Kf.tol;  tol_p.Muw  = p.Muw.tol;

% Stabilizer calculations
[stab_flow, stab_comp, stab_ceq] = colsys_stab(stab_x, stab_d, stab_p);

% Benzene column calculations
ben_F  = stab_flow(4);                                % Bottom flow rate from stabilizer
ben_qF = 1;                                           % Assuming liquid feed to benzene column
ben_zF = stab_comp(2,3:end)/sum(stab_comp(2,3:end));  % Bottom composition from stabilizer assuming only benzene, toluene and diphenyl
ben_d  = [ben_F ben_qF ben_zF];
[ben_flow, ben_comp, ben_ceq] = colsys_ben(ben_x, ben_d, ben_p);

% Toluene column calculations
tol_F  = ben_flow(4);                       % Bottom flow rate from benzene column
tol_qF = 1;                                 % Assuming liquid feed to toluene column
tol_zF = ben_comp(2,:)/sum(ben_comp(2,:));  % Bottom composition from benzene column
tol_d  = [tol_F tol_qF tol_zF];
[tol_flow, tol_comp, tol_ceq] = colsys_tol(tol_x, tol_d, tol_p);

% Outputs
ben_comp = [zeros(2,2) ben_comp];
tol_comp = [zeros(2,2) tol_comp];
L        = [stab_flow(1) ben_flow(1) tol_flow(1)];
V        = [stab_flow(2) ben_flow(2) tol_flow(2)];
D        = [stab_flow(3) ben_flow(3) tol_flow(3)];
B        = [stab_flow(4) ben_flow(4) tol_flow(4)];
xD       = [stab_comp(1,:); ben_comp(1,:); tol_comp(1,:)];
xB       = [stab_comp(2,:); ben_comp(2,:); tol_comp(2,:)];
dMxdt    = [stab_ceq; ben_ceq; tol_ceq];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Columns %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [stab_flow, stab_comp, stab_ceq] = colsys_stab(stab_x, stab_d, stab_p)

% Number of stages (including reboiler and total condenser: 
NT = stab_p.ntnfnc(1);

% Location of feed stage (stages are counted starting from the bottom):
NF = stab_p.ntnfnc(2);

% Number of components
NC = stab_p.ntnfnc(3);

% Relative volatilities: alpha=[alpha1 alpha2...alphaNC]
alpha = stab_p.alpha;

% Coefficients for the Francis weir formula
Kf = stab_p.Kf;

% Liquid holdup below weir
Muw = stab_p.Muw;

% Inputs and disturbances
% Inputs
LT = stab_x(NC*NT+2); % Reflux
VB = stab_x(NC*NT+3); % Boilup
D  = stab_x(NC*NT+4); % Distillate
xB = stab_x(NC*NT+5); % Bottoms
% Disturbances
F  = stab_d(1);       % Feedrate
qF = stab_d(2);       % Feed liquid fraction
zF = stab_d(3:end);   % Feed compositions

% THE MODEL
X = stab_x;
B = stab_x(NT*NC+1);
overcool = 34;
modelcolumn
dXdt = [dXdt; x(1,2) - xB];

% Outputs
stab_flow = [LT VB D B]';
stab_comp = [x(NT,:); x(1,:)]; % That is, stab_comp = [xD; xB] and this is 2xNC vector
stab_ceq  = dXdt;

function [ben_flow, ben_comp, ben_ceq] = colsys_ben(ben_x, ben_d, ben_p)

% Number of stages (including reboiler and total condenser: 
NT = ben_p.ntnfnc(1);

% Location of feed stage (stages are counted starting from the bottom):
NF = ben_p.ntnfnc(2);

% Number of components
NC = ben_p.ntnfnc(3);

% Relative volatilities: alpha=[alpha1 alpha2...alphaNC]
alpha = ben_p.alpha;

% Coefficients for the Francis weir formula
Kf = ben_p.Kf;

% Liquid holdup below weir
Muw = ben_p.Muw;

% Inputs and disturbances
% Inputs
LT = ben_x(NC*NT+2); % Reflux
VB = ben_x(NC*NT+3); % Boilup
D  = ben_x(NC*NT+4); % Distillate
xD = ben_x(NC*NT+5); % Bottoms
% Disturbances
F  = ben_d(1);       % Feedrate
qF = ben_d(2);       % Feed liquid fraction
zF = ben_d(3:end);   % Feed compositions

% THE MODEL
X = ben_x;
B = ben_x(NT*NC+1);
overcool = 24;
modelcolumn
dXdt = [dXdt; x(NT,1) - xD];

% Outputs
ben_flow = [LT VB D B]';
ben_comp = [x(NT,:); x(1,:)]; % That is, stab_comp = [xD; xB] and this is 2xNC vector
ben_ceq  = dXdt;

function [tol_flow, tol_comp, tol_ceq] = colsys_tol(tol_x, tol_d, tol_p)

% Number of stages (including reboiler and total condenser: 
NT = tol_p.ntnfnc(1);

% Location of feed stage (stages are counted starting from the bottom):
NF = tol_p.ntnfnc(2);

% Number of components
NC = tol_p.ntnfnc(3);

% Relative volatilities: alpha=[alpha1 alpha2...alphaNC]
alpha = tol_p.alpha;

% Coefficients for the Francis weir formula
Kf = tol_p.Kf;

% Liquid holdup below weir
Muw = tol_p.Muw;

% Inputs and disturbances
% Inputs
LT = tol_x(NC*NT+2); % Reflux
VB = tol_x(NC*NT+3); % Boilup
D  = tol_x(NC*NT+4); % Distillate
xD = tol_x(NC*NT+5); % Bottoms
% Disturbances
F  = tol_d(1);       % Feedrate
qF = tol_d(2);       % Feed liquid fraction
zF = tol_d(3:end);   % Feed compositions

% THE MODEL
X = tol_x;
B = tol_x(NT*NC+1);
overcool = 19;
modelcolumn
dXdt = [dXdt; x(NT,2) - xD];

% Outputs
tol_flow = [LT VB D B]';
tol_comp = [x(NT,:); x(1,:)]; % That is, stab_comp = [xD; xB] and this is 2xNC vector
tol_ceq  = dXdt;


