function [dXdt,df,Fl,Fv,P,Pl] = flash(alg,diff,Fi,kv,kl,kf,Pdownliq,cvfsep);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% flash - physical data out of flash and time gradient
%
% Inputs:    alg  - algebraic variables   
%            diff - differential variables
%            Fi   - separator inlet flow  [struct]
%
% Outputs:   dXdt - state gradients
%            Fl   - outlet liquid flow    [struct]       
%            Fv   - outlet vapor flow     [struct]
%            P    - pressure separator    [psia]
%          
% Written by   : Marius S. Govatsmark        19.sept.1998
% Last modified: Marius S. Govatsmark        22.aug.1999
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% data
   diam = 3.948; % diameter [ft]
   heat = 0;     % adiabtic
 
% Number of components
   nc = length(Fi.F);

% differential variables
   H = diff(1:nc); % total molar holdup by component in flash (vapor + liquid phase)
   E = diff(nc+1); % total energy holdup in flash
      
% algebraic variables
   alpha = alg(1); % ratio between vapor and liquid holdup (Hv/Hl) in the flash
   x     = alg(2:nc+1); % liquid composition out of flash
   T     = alg(nc+2); % flash temperature
   Pgas  = alg(nc+3); % pressure after the vapor valve out of flash (before the purge split). This is assumed controlled!

% size
   ar = pi()*diam^2/4;
   heigt = 6*diam;
   Vol = heigt*ar;
   
% define struct
   Fv = stream;
   Fl = Fv;
   
% pressure in separator tank
   DP = (sum(Fi.F)/(kf*cvfsep))^2; % pressure drop due to the upstream valve
   P = Fi.P - DP;
   Fl.P = P;
   Fv.P = P;
   
% Vapor composition (phase-equilirium)
   K = kvalue(T)';
   for i = 1:nc
     y(i) = K(i)*x(i);
   end
   y = y(:);

% Density
   Dl = density(T,P,x,'LIQ');
   Dv = density(T,P,y,'VAP');
   
% Molweight - liquid & vapor phase
   Ml = molweight(x);
   Mv = molweight(y);

% Total molar holdup
   Htot = sum(H);
   
% Liquid level
   LiqLevel = Ml*Htot*(1 - alpha)/(Dl*ar);
      
% Pressure - front of valve   
   Pl = P + 32.2*Dl*LiqLevel;
   
% Vapor and liquid flow   
   V = kv*(P - Pgas)^0.5;
   L = kl*(Pl - Pdownliq)^0.5;

if P < Pgas
    disp(sprintf('Separator vapor pressure (P) is less than Pgas: P = %0.4g', P));
end

if Pl < Pdownliq
    disp(' ')
    disp(sprintf('Separator liquid pressure (Pl) is less than Pdownliq: Pl = %0.4g', Pl));
    disp(sprintf('Separator vapor pressure: P = %0.4g', P));
end

% Liquid and vapor flow
   Fl.F = L*x;
   Fv.F = V*y;

% Entalphy - liquid & vapor
   Fl.T = T;
   Fv.T = T;
   Fl.E = enthalpy(Fl);
   Fv.E = enthalpy(Fv);

% Balance equations / differential equations
   dHdt = Fi.F - Fv.F - Fl.F;
   dEdt = Fi.E - Fv.E - Fl.E;

   dXdt = [dHdt; dEdt];

% Internal equations
   df1  = sum(x) - sum(y);
   df26 = -H/Htot + alpha*y + (1 - alpha)*x;
   df7  = -E/Htot + alpha*Fv.E/sum(Fv.F) + (1 - alpha)*Fl.E/sum(Fl.F);
   df8  = -Vol/Htot + alpha*Mv/Dv + (1 - alpha)*Ml/Dl;
   
   df = [df1; df26; df7; df8];      
   
% Gamma - adiabatic pressure equation
   G = gamma1(T,y);
   
% Liquid stream (after valve)
   Fl.P = Pdownliq;

% Vapor stream (after valve)
   Fv.P = Pgas;
   Fv.T = (T + 459.67)*(Fv.P/P)^G - 459.67;
