function [Fout,dCdt,DF,F,E,F4,E0]=reactor(Fi,F,Ct,T,x)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% reactor - calculates physical data after reactor 
%
% Inputs:    Fi  - flow out of Furnace      [struct]
%            F   - flow out of Reactor      [struct]
%            Ct  - concentration of toluene [struct]
%            T   - temperature profile      [struct]
%            x   - conversion of toluene    [struct]
%
% Outputs:   Fout  - outlet flow                                    [struct]
%            dCdt  - dynamic conc. of toluene                       [struct]
%            F     - flow profile through the reactor               [struct]
%            E     - energy profile through the reactor (calc. 2)   [struct]
%            F4    - mass flow rate through the reactor             [struct]
%            E0    - energy profile through the reactor (calc. 1)   [struct]
%
%
%Written by   : Marius S. Govatsmark       18.aug.1998
%Last modified: Marius S. Govatsmark       22.aug.1999
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%msg19990825

n=length(x);
Fmatr=Fi.F';
Ematr=Fi.E;
E0=[];
Tmatr=[Fi.T; T];

%selectivity
x0=1-0.0036^(1/1.544);
p=x<x0;
s1=((1-x).^1.544-0.0036)./(1-x).^1.544;
s2=-0.3/0.005*(x-x0);
s=p.*s1+(1-p).*s2;
if max(x)>0.40   
   disp(sprintf('max(x)=%0.4g',max(x)));
end

for k=1:n
   F.T=T(k);
   F.F(4)=Fi.F(4)*(1-x(k));                                         % toluene
   F.F(3)=Fi.F(3)*(2*s(k)-1)/s(k)+Fi.F(4)*s(k)*x(k);                % benzene
   F.F(5)=Fi.F(5)+Fi.F(4)*x(k)*(1-s(k))/2+Fi.F(3)*(1-s(k))/(2*s(k));% diphenyl
   F.F(1)=Fi.F(1)-Fi.F(4)*x(k)*(1+s(k))/2+Fi.F(3)*(1-s(k))/(2*s(k));% hydrogen
%    F.F(2)=Fi.F(2)+Fi.F(4)*s(k)*x(k);                                % methane (selectivity included. Original from Brognaux, C. (1992). A case study in operability analysis: the HDA plant, Master's Thesis, University of London)
   F.F(2)=Fi.F(2)+Fi.F(4)*x(k);                                     % methane (no selectivity included. Corrected from Cao, Y., Rossiter, D., Edwards, D. W., Knechtel, J., Owens D. (1998). "Modelling issues for control structure selection in a chemical process", Computers and Chemical Engineering, vol 22, Suppl. pp. S411-S418)

   F.E=Fi.E+0.021530*x(k)*Fi.F(4);
   E0=[E0; enthalpy(F)]; % enthalpy profile through the reactor (calculation 1)
   Fmatr=[Fmatr; F.F']; % flow rate profile through the reactor
   Ematr=[Ematr; F.E]; % enthalpy profile through the reactor (calculation 2)
   Fi=F;     
end
Fout=F;

Ftotmatr=sum(Fmatr')';
FTOT=[Ftotmatr Ftotmatr Ftotmatr Ftotmatr Ftotmatr];
ymatr=Fmatr./FTOT;

%density
yi=ymatr(1:n,:);
Ti=Tmatr(1:n);
D=F.P*(0.18*yi(:,1)+1.47*yi(:,2)+7.17*yi(:,3)+8.46*yi(:,4)+14.16*yi(:,5));
D=D./(459.69+Ti);

%molweight
M=2*yi(:,1)+16*yi(:,2)+78*yi(:,3)+92*yi(:,4)+154*yi(:,5);

%Differences
F=Fmatr(2:n+1,:);
E=Ematr(2:n+1);
Ftoti=Ftotmatr(1:n);
F4=Ct.*Ftoti.*M./D;
dF4=F(:,4)-F4;
dE=E-E0;
DF=zeros(2*n,1);
t1=[1:n]*2-1;
t2=[1:n]*2;
DF(t1)=dF4; 
DF(t2)=dE;

%gradients
Fi=Fmatr(1:n,:);
%k=2.522e11*exp(-47136.12./(T+459.67));
k=1.832e11*exp(-47136.12./(T+459.67)); %2.522 (original k0); 3.022 (value of k0 to get 0.75 conversion with compressor bhp = 591hp); 1.832  (value of k0 to get 0.75 conversion with compressor bhp = 334hp, according to Douglas!)
k=k(:);
Ch=D.*F(:,1)./(Ftoti.*M);
dCdt=(Fi(:,4)-F(:,4)-3600*k*470.*Ct.*sqrt(Ch))/470;
